"""Session interfaces."""
import os

from pdpyras import APISession  # type: ignore


#: PD API key
API_KEY = os.environ.get("PD_ACCOUNT_TOKEN")

#: API session
_API_SESSION = None


def get_api_session():
    """Get api session."""
    global _API_SESSION  # pylint: disable=global-statement

    if not _API_SESSION:
        _API_SESSION = APISession(API_KEY)

    return _API_SESSION



"""Implement pd user command."""
import json

from ..api.user import get_user, ls_user


def user(user_id: str = None) -> str:
    """Get user.

    :param user_id: user id. "me" for current user.
        list all user if unspecified.
    """
    result = get_user(user_id=user_id) if user_id else ls_user()

    return json.dumps(result)



"""Interface to user endpoint."""
import logging
from typing import List

from .session import get_api_session

LOGGER = logging.getLogger(__name__)


def get_user(*, user_id: str) -> dict:
    """Get user."""
    session = get_api_session()
    return session.rget(f"/users/{user_id}")


def ls_user() -> List:
    """List users."""
    session = get_api_session()
    return session.list_all("users")



"""Implement pd ls command."""
import datetime
from typing import List
import json
from csv import DictWriter
import io

from ..api.incident import Status, Urgency, list_incidents


def ls(  # pylint: disable=invalid-name
    since: str = None,
    statuses: List[str] = None,
    user_ids: List[str] = None,
    urgency: str = None,
    column: bool = False,
    delimiter: str = "\t",
) -> str:
    """List incidents.

    :param since: since date
    :param statuses: filter by statuses
    :param user_ids: filter by user id
    :param urgency: filter by urgency
    :param column: return rows of columns, instead of json string
    :param delimiter: delimiter used for column output
    """
    # pylint: disable=too-many-arguments
    if statuses:
        if isinstance(statuses, str):
            statuses = [statuses]
    else:
        statuses = []

    if user_ids:
        if isinstance(user_ids, str):
            user_ids = [user_ids]
    else:
        user_ids = []

    incidents = list_incidents(
        since=datetime.date.fromisoformat(since) if since else None,
        statuses=[Status(status) for status in statuses],
        user_ids=user_ids,
        urgency=Urgency(urgency) if urgency else None,
    )

    if column:
        return _as_columns(
            incidents=incidents,
            columns=[
                "status",
                "urgency",
                "title",
                "created_at",
                "service",
                "assignments",
            ],
            delimiter=delimiter,
        )

    return json.dumps(incidents)


def _as_columns(*, incidents, columns, delimiter):

    column_transforms = {
        "service": lambda x: x["summary"],
        "assignments": lambda x: ", ".join(
            assignment["assignee"]["summary"] for assignment in x
        ),
    }

    def filter_fields(incident):
        row = {}

        for column in columns:
            if column not in incident.keys():
                continue
            value = incident[column]
            if transform := column_transforms.get(column):
                value = transform(value)
            row[column] = value

        return row

    rows = [filter_fields(incident) for incident in incidents]

    with io.StringIO() as file_:
        writer = DictWriter(file_, fieldnames=columns, delimiter=delimiter)
        writer.writeheader()
        writer.writerows(rows)

        return file_.getvalue()



"""Interface to incident endpoint."""
import logging
import datetime
from typing import Optional, List, Dict, Any
from enum import Enum

from .session import get_api_session

LOGGER = logging.getLogger(__name__)


class Status(Enum):
    """Incident status."""

    TRIGGERED = "triggered"
    ACKNOWLEDGED = "acknowledged"
    RESOLVED = "resolved"


class Urgency(Enum):
    """Incident urgency."""

    HIGH = "high"
    LOW = "low"


def list_incidents(
    *,
    since: Optional[datetime.date] = None,
    statuses: Optional[List[Status]] = None,
    user_ids: Optional[List[str]] = None,
    urgency: Optional[Urgency] = None,
) -> List:
    """List incidents."""
    params: Dict[str, Any] = {}
    if since:
        params["since"] = since.isoformat()

    if statuses:
        params["statuses"] = [status.value for status in statuses]

    if user_ids:
        params["user_ids"] = user_ids

    if urgency:
        params["urgencies"] = [urgency.value]

    LOGGER.debug(f"{params=}")

    session = get_api_session()
    incidents = session.list_all("incidents", params=params)

    return incidents



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['pdcli', 'pdcli.api', 'pdcli.command']

package_data = \
{'': ['*']}

install_requires = \
['fire>=0.4.0,<0.5.0', 'pdpyras>=4.4.0,<5.0.0']

entry_points = \
{'console_scripts': ['pd = pdcli.cli:pd']}

setup_kwargs = {
    'name': 'python-pdcli',
    'version': '0.0.3',
    'description': 'PD cli',
    'long_description': '# PD cli\n\n\n## Test\n```\nexport PD_ACCOUNT_TOKEN=xyz\n\npd ls \\\n  --statuses=acknowledged,triggered --since=$(date -v -1d +%F) --column \\\n  | column -t -s$\'\\t\'\n```\n\nDefault output is json string. In order to extract field:\n```shell\n# to extract "id" from user record\npd user --user-id=me | jq -r .id\n```\n\n\n## Reference\n* https://pypi.org/project/pdpyras/\n* https://developer.pagerduty.com/api-reference/\n',
    'author': 'King-On Yeung',
    'author_email': 'koyeung@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/koyeung/python-pdcli',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
